@@ -1,7 +1,10 @@
-from mock import patch
 import unittest
-from ovh_iplb import iplb_client
+
 import ovh
+from ansible.errors import AnsibleError
+from mock import patch
+
+from ovh_iplb import iplb_client
 
 
 class TestRealPayloadOkWithWantedPayload(unittest.TestCase):
@@ -210,6 +213,39 @@ class TestFrontend(PrepareMockedIPLB, unittest.TestCase):
                                                            displayName='name',
                                                            allowedSource=['9.8.4.4/28', '10.8.4.4/28'])
 
+    def test_update_when_allowed_source_are_not_returned_by_server(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/tcp/frontend/42': {'frontendId': 42, 'displayName': 'name', 'type': 'tcp'},
+            '/ipLoadbalancing/iplb_id/tcp/frontend/41': {'frontendId': 41, 'displayName': 'otherName'},
+            '/ipLoadbalancing/iplb_id/tcp/frontend': ['41', '42']
+        })
+
+        self.assertTrue(self.iplb.frontend({
+            'type': 'tcp',
+            'name': 'name',
+            'allowed_source': ['9.8.4.4/28', '10.8.4.4/28']
+        }).update())
+        self.mocked_client_instance.put.assert_called_with('/ipLoadbalancing/iplb_id/tcp/frontend/42',
+                                                           displayName='name',
+                                                           allowedSource=['9.8.4.4/28', '10.8.4.4/28'])
+
+    def test_update_when_allowed_source_are_not_returned_by_server_but_also_not_defined(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/tcp/frontend/42': {'frontendId': 42,
+                                                         'displayName': 'name',
+                                                         'type': 'tcp',
+                                                         'allowedSource': None},
+            '/ipLoadbalancing/iplb_id/tcp/frontend/41': {'frontendId': 41, 'displayName': 'otherName'},
+            '/ipLoadbalancing/iplb_id/tcp/frontend': ['41', '42']
+        })
+
+        self.assertFalse(self.iplb.frontend({
+            'type': 'tcp',
+            'name': 'name',
+            'allowed_source': []
+        }).update())
+        self.mocked_client_instance.put.assert_not_called()
+
     def test_exist_with_unexisting_name(self):
         self.mock_get({
             '/ipLoadbalancing/iplb_id/tcp/frontend/42': {'frontendId': 42, 'displayName': 'name'},
@@ -361,6 +397,100 @@ class TestFrontend(PrepareMockedIPLB, unittest.TestCase):
                 'zone': 'all'
                 }).body()
 
+    def test_body_with_default_ssl_id(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/tcp/farm': ['42'],
+            '/ipLoadbalancing/iplb_id/tcp/farm/42': {'farmId': 42, 'displayName': 'name'}
+        })
+        self.assertEqual(
+            self.iplb.frontend({
+                'id': '123456789',
+                'type': 'http',
+                'default_ssl_id': 666,
+                'name': 'name',
+                'port': '666',
+                'ssl': True,
+                'zone': 'all'
+            }).body(),
+            {
+                'displayName': 'name',
+                'port': '666',
+                'zone': 'all',
+                'defaultSslId': 666,
+                'ssl': True
+            }
+        )
+
+    def test_body_with_ssl_id_fetched_by_common_name(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/ssl': [10056, 10057, 10058],
+            '/ipLoadbalancing/iplb_id/ssl/10056': {'id': 10056, 'subject': 'CN=choo.bar'},
+            '/ipLoadbalancing/iplb_id/ssl/10057': {'id': 10057, 'subject': 'CN=foo.bar'},
+            '/ipLoadbalancing/iplb_id/ssl/10058': {'id': 10058, 'subject': 'CN=foo.barr'},
+        })
+        self.assertEqual(
+            self.iplb.frontend({
+                'id': '123456789',
+                'type': 'http',
+                'ssl_certificate_common_name': 'foo.bar',
+                'name': 'name',
+                'port': '666',
+                'ssl': True,
+                'zone': 'all'
+            }).body(),
+            {
+                'displayName': 'name',
+                'port': '666',
+                'zone': 'all',
+                'defaultSslId': 10057,
+                'ssl': True
+            }
+        )
+
+    def test_body_with_ssl_id_fetched_by_common_name_no_match(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/ssl': [10056, 10057, 10058],
+            '/ipLoadbalancing/iplb_id/ssl/10056': {'id': 10056, 'subject': 'CN=choo.bar'},
+            '/ipLoadbalancing/iplb_id/ssl/10057': {'id': 10057, 'subject': 'CN=foo.baar'},
+            '/ipLoadbalancing/iplb_id/ssl/10058': {'id': 10058, 'subject': 'CN=foo.barr'},
+        })
+        with self.assertRaises(AnsibleError):
+            self.iplb.frontend({
+                'id': '123456789',
+                'type': 'http',
+                'ssl_certificate_common_name': 'foo.bar',
+                'name': 'name',
+                'port': '666',
+                'ssl': True,
+                'zone': 'all'
+            }).body()
+
+    def test_body_with_ssl_id_fetched_by_common_name_multiple_matches(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/ssl': [10056, 10058, 10057],
+            '/ipLoadbalancing/iplb_id/ssl/10056': {'id': 10056, 'subject': 'CN=choo.bar'},
+            '/ipLoadbalancing/iplb_id/ssl/10057': {'id': 10057, 'subject': 'CN=foo.bar'},
+            '/ipLoadbalancing/iplb_id/ssl/10058': {'id': 10058, 'subject': 'CN=foo.bar'},
+        })
+        self.assertEqual(
+            self.iplb.frontend({
+                'id': '123456789',
+                'type': 'http',
+                'ssl_certificate_common_name': 'foo.bar',
+                'name': 'name',
+                'port': '666',
+                'ssl': True,
+                'zone': 'all'
+            }).body(),
+            {
+                'displayName': 'name',
+                'port': '666',
+                'zone': 'all',
+                'defaultSslId': 10058,
+                'ssl': True
+            }
+        )
+
     def test_read(self):
         self.mock_get({
             '/ipLoadbalancing/iplb_id/tcp/frontend/42': {'frontendId': 42, 'displayName': 'name', 'field': 'data'},
@@ -542,8 +672,22 @@ class TestFarm(PrepareMockedIPLB, unittest.TestCase):
                                         'zone': 'gra',
                                         'servers': [{'address': '8.8.8.8'}]}).apply())
 
+    def test_apply_with_more_than_100_servers(self):
+        self.mock_get({
+            '/ipLoadbalancing/iplb_id/tcp/farm/42/server': [],
+            '/ipLoadbalancing/iplb_id/tcp/farm/42': {'farmId': 42, 'displayName': 'name', 'zone': 'gra'},
+            '/ipLoadbalancing/iplb_id/tcp/farm': ['42'],
+        })
+
+        self.assertTrue(self.iplb.farm({'type': 'tcp',
+                                        'name':
+                                        'name',
+                                        'zone': 'gra',
+                                        'servers': [{'address': '8.8.8.{}'.format(i)} for i in range(120)]}).apply())
+
         self.mocked_client_instance.post.assert_called_with('/ipLoadbalancing/iplb_id/tcp/farm/42/server',
-                                                            address='8.8.8.8')
+                                                            address='8.8.8.99')
+        self.assertEqual(self.mocked_client_instance.post.call_count, 100)
 
     def test_apply_with_servers_to_update(self):
         self.mock_get({
