@@ -1,8 +1,11 @@
 import re
+import six
 
 
 def convert_to_str_if_unicode(val):
-    return str(val) if isinstance(val, unicode) else val
+    if isinstance(val, six.text_type):
+        return six.ensure_str(val)
+    return val
 
 
 def clean_unicode_of_dict(d):
